queueconfig

package
v0.1.6 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 16, 2026 License: MIT Imports: 1 Imported by: 8

Documentation

Overview

Package queueconfig contains shared configuration structs used by optional driver modules.

This is an advanced support package for driver constructors. Most application code should use package queue and the driver modules directly and does not need to import queueconfig.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type DriverBaseConfig

type DriverBaseConfig struct {
	DefaultQueue string
	Observer     queue.Observer
	Logger       queue.Logger
}

DriverBaseConfig contains common constructor options shared by driver modules. It is embedded by driver module config types (for example, redisqueue.Config).

Jump to